package SATE;

import java.awt.Point;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import marf.Storage.Database;
import marf.Storage.IDatabaseConnection;
import marf.Storage.StorageException;
import marf.util.Arrays;
import marf.util.Debug;
import Storage.CWE;
import Storage.FileItem;
import Storage.GenericMetaXMLLoader;
import Storage.MetaLocation;


/**
 * <p>Class SATEDb manages database of subjects at the application level.</p>
 * 
 * @author Serguei Mokhov
 * @version $Id: SATEDb.java,v 1.14 2011/01/14 00:31:30 mokhov Exp $
 * @since XXX
 */
public class SATEDb
extends Database
{
	/**
	 * CVE -> ID.
	 */
	public static final HashMap<String,Integer> MAP_CVES_TO_IDS = new HashMap<String,Integer>();

	/**
	 * CWEID -> ID.
	 */
	public static final HashMap<String,Integer> MAP_CWEIDS_TO_IDS = new HashMap<String,Integer>();

	/**
	 * CEWID -> Name. 
	 */
	public static final HashMap<String,String> MAP_CWEIDS_TO_NAMES = new HashMap<String,String>();

	/**
	 * CVE -> CWE.
	 */
	public static final HashMap<String,String> MAP_CVES_TO_CWES = new HashMap<String,String>();

	/**
	 * CWE -> Severity.
	 */
	public static final HashMap<String,Integer> MAP_CWES_TO_SEVERITIES = new HashMap<String,Integer>();
	
	public static final HashMap<String,String> MAP_LETTERS_TO_JOURNALS = new HashMap<String,String>();
	
	static
	{
		MAP_CWEIDS_TO_IDS.put("NVD-CWE-DesignError", 997);
		MAP_CWEIDS_TO_IDS.put("NVD-CWE-Other", 998);
		MAP_CWEIDS_TO_IDS.put("NVD-CWE-noinfo", 999);
		MAP_CWEIDS_TO_IDS.put("CWE-16", 16);
		MAP_CWEIDS_TO_IDS.put("CWE-20", 20);
		MAP_CWEIDS_TO_IDS.put("CWE-22", 22);
		MAP_CWEIDS_TO_IDS.put("CWE-79", 79);
		MAP_CWEIDS_TO_IDS.put("CWE-94", 94);
		MAP_CWEIDS_TO_IDS.put("CWE-119", 119);
		MAP_CWEIDS_TO_IDS.put("CWE-189", 189);
		MAP_CWEIDS_TO_IDS.put("CWE-200", 200);
		MAP_CWEIDS_TO_IDS.put("CWE-255", 255);
		MAP_CWEIDS_TO_IDS.put("CWE-264", 264);
		MAP_CWEIDS_TO_IDS.put("CWE-399", 399);

		MAP_CWEIDS_TO_NAMES.put("NVD-CWE-Other", "Other");
		MAP_CWEIDS_TO_NAMES.put("NVD-CWE-noinfo", "Insufficient Information");
		MAP_CWEIDS_TO_NAMES.put("CWE-16", "Configuration");
		MAP_CWEIDS_TO_NAMES.put("CWE-20", "Input Validation");
		MAP_CWEIDS_TO_NAMES.put("CWE-22", "Path Traversal");
		MAP_CWEIDS_TO_NAMES.put("CWE-79", "Cross-Site Scripting (XSS)");
		MAP_CWEIDS_TO_NAMES.put("CWE-94", "Code Injection");
		MAP_CWEIDS_TO_NAMES.put("CWE-119", "Buffer Errors");
		MAP_CWEIDS_TO_NAMES.put("CWE-189", "Numeric Errors");
		MAP_CWEIDS_TO_NAMES.put("CWE-200", "Information Leak / Disclosure");
		MAP_CWEIDS_TO_NAMES.put("CWE-255", "Credentials Management");
		MAP_CWEIDS_TO_NAMES.put("CWE-264", "Permissions, Privileges, and Access Control (CWE-264)");
		MAP_CWEIDS_TO_NAMES.put("CWE-399", "Resource Management Errors");

		MAP_CWES_TO_SEVERITIES.put("NVD-CWE-DesignError", 2);
		MAP_CWES_TO_SEVERITIES.put("NVD-CWE-Other", 5);
		MAP_CWES_TO_SEVERITIES.put("NVD-CWE-noinfo", 5);
		MAP_CWES_TO_SEVERITIES.put("CWE-16", 3);
		MAP_CWES_TO_SEVERITIES.put("CWE-20", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-22", 2);
		MAP_CWES_TO_SEVERITIES.put("CWE-79", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-94", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-119", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-189", 2);
		MAP_CWES_TO_SEVERITIES.put("CWE-200", 2);
		MAP_CWES_TO_SEVERITIES.put("CWE-255", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-264", 1);
		MAP_CWES_TO_SEVERITIES.put("CWE-399", 3);
		

	
		/*
		Wireshark:
		CVE-2010-2287  	Buffer Errors (CWE-119)
		CVE-2010-2286 	Resource Management Errors (CWE-399)
		CVE-2010-2285 	Other (NVD-CWE-Other)
		CVE-2010-2284 	Buffer Errors (CWE-119)
		CVE-2010-1455 	Input Validation (CWE-20)
		CVE-2010-0304 	Buffer Errors (CWE-119)
		CVE-2009-4378 	Other (NVD-CWE-Other)
		CVE-2009-4377 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2009-4376 	Buffer Errors (CWE-119)
		CVE-2009-3829 	Numeric Errors (CWE-189)
		CVE-2009-3551 	Numeric Errors (CWE-189)
		CVE-2009-3550 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2009-3549 	Input Validation (CWE-20)
		CVE-2009-3243 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2009-3242 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2009-3241 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2009-2559 	Buffer Errors (CWE-119)
		*/
		
		MAP_CVES_TO_IDS.put("Unknown", 0);
		
		// Wireshark
		MAP_CVES_TO_IDS.put("CVE-2010-2287", 1);
		MAP_CVES_TO_IDS.put("CVE-2010-2286", 2);
		MAP_CVES_TO_IDS.put("CVE-2010-2285", 3);
		MAP_CVES_TO_IDS.put("CVE-2010-2284", 4);
		MAP_CVES_TO_IDS.put("CVE-2010-2283", 20102283);
		MAP_CVES_TO_IDS.put("CVE-2010-1455", 5);
		MAP_CVES_TO_IDS.put("CVE-2010-0304", 6);
		MAP_CVES_TO_IDS.put("CVE-2009-4378", 7);
		MAP_CVES_TO_IDS.put("CVE-2009-4377", 8);
		MAP_CVES_TO_IDS.put("CVE-2009-4376", 9);
		MAP_CVES_TO_IDS.put("CVE-2009-3829", 10);
		MAP_CVES_TO_IDS.put("CVE-2009-3551", 11);
		MAP_CVES_TO_IDS.put("CVE-2009-3550", 12);
		MAP_CVES_TO_IDS.put("CVE-2009-3549", 13);
		MAP_CVES_TO_IDS.put("CVE-2009-3243", 14);
		MAP_CVES_TO_IDS.put("CVE-2009-3242", 15);
		MAP_CVES_TO_IDS.put("CVE-2009-3241", 16);
		MAP_CVES_TO_IDS.put("CVE-2009-2563", 20092563);
		MAP_CVES_TO_IDS.put("CVE-2009-2562", 20092562);
		MAP_CVES_TO_IDS.put("CVE-2009-2561", 20092561);
		MAP_CVES_TO_IDS.put("CVE-2009-2560", 20092560);
		MAP_CVES_TO_IDS.put("CVE-2009-2559", 17);

		MAP_CVES_TO_CWES.put("CVE-2010-2287", "CWE-119");
		MAP_CVES_TO_CWES.put("CVE-2010-2286", "CWE-399");
		MAP_CVES_TO_CWES.put("CVE-2010-2285", "NVD-CWE-Other");
		MAP_CVES_TO_CWES.put("CVE-2010-2284", "CWE-119");
		MAP_CVES_TO_CWES.put("CVE-2010-2283", "NVD-CWE-Other");
		MAP_CVES_TO_CWES.put("CVE-2010-1455", "CWE-20");
		MAP_CVES_TO_CWES.put("CVE-2010-0304", "CWE-119");
		MAP_CVES_TO_CWES.put("CVE-2009-4378", "NVD-CWE-Other");
		MAP_CVES_TO_CWES.put("CVE-2009-4377", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-4376", "CWE-119");
		MAP_CVES_TO_CWES.put("CVE-2009-3829", "CWE-189");
		MAP_CVES_TO_CWES.put("CVE-2009-3551", "CWE-189");
		MAP_CVES_TO_CWES.put("CVE-2009-3550", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-3549", "CWE-20");
		MAP_CVES_TO_CWES.put("CVE-2009-3243", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-3242", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-3241", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-2563", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-2562", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-2561", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-2560", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2009-2559", "CWE-119");
		
		/*
		Chrome:
		CVE-2010-2304  	Buffer Errors (CWE-119)
		CVE-2010-2303 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2010-2302 	Resource Management Errors (CWE-399)
		CVE-2010-2301 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2010-2300 	Resource Management Errors (CWE-399)
		CVE-2010-2299 	Code Injection (CWE-94)
		CVE-2010-2298 	Input Validation (CWE-20)
		CVE-2010-2297 	Code Injection (CWE-94)
		CVE-2010-2296 	Permissions, Privileges, and Access Control (CWE-264)
		CVE-2010-2295 	Other (NVD-CWE-Other)
		CVE-2010-2110 	Design Error (NVD-CWE-DesignError)
		CVE-2010-2109 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2010-2108 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2010-2107 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2010-2106 	Insufficient Information (NVD-CWE-noinfo)
		CVE-2010-2105 	Design Error (NVD-CWE-DesignError)
		CVE-2010-1770 	Code Injection (CWE-94)
		*/

		// Chrome
		MAP_CVES_TO_IDS.put("CVE-2010-2304", 18);
		MAP_CVES_TO_IDS.put("CVE-2010-2303", 19);
		MAP_CVES_TO_IDS.put("CVE-2010-2302", 20);
		MAP_CVES_TO_IDS.put("CVE-2010-2301", 21);
		MAP_CVES_TO_IDS.put("CVE-2010-2300", 22);
		MAP_CVES_TO_IDS.put("CVE-2010-2299", 23);
		MAP_CVES_TO_IDS.put("CVE-2010-2298", 24);
		MAP_CVES_TO_IDS.put("CVE-2010-2297", 25);
		MAP_CVES_TO_IDS.put("CVE-2010-2296", 26);
		MAP_CVES_TO_IDS.put("CVE-2010-2295", 27);
		MAP_CVES_TO_IDS.put("CVE-2010-2110", 28);
		MAP_CVES_TO_IDS.put("CVE-2010-2109", 29);
		MAP_CVES_TO_IDS.put("CVE-2010-2108", 30);
		MAP_CVES_TO_IDS.put("CVE-2010-2107", 31);
		MAP_CVES_TO_IDS.put("CVE-2010-2106", 32);
		MAP_CVES_TO_IDS.put("CVE-2010-2105", 33);
		MAP_CVES_TO_IDS.put("CVE-2010-1770", 34);

		MAP_CVES_TO_CWES.put("CVE-2010-2304", "CWE-119");
		MAP_CVES_TO_CWES.put("CVE-2010-2303", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2010-2302", "CWE-399");
		MAP_CVES_TO_CWES.put("CVE-2010-2301", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2010-2300", "CWE-399");
		MAP_CVES_TO_CWES.put("CVE-2010-2299", "CWE-94");
		MAP_CVES_TO_CWES.put("CVE-2010-2298", "CWE-20");
		MAP_CVES_TO_CWES.put("CVE-2010-2297", "CWE-94");
		MAP_CVES_TO_CWES.put("CVE-2010-2296", "CWE-264");
		MAP_CVES_TO_CWES.put("CVE-2010-2295", "NVD-CWE-Other");
		MAP_CVES_TO_CWES.put("CVE-2010-2110", "NVD-CWE-DesignError");
		MAP_CVES_TO_CWES.put("CVE-2010-2109", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2010-2108", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2010-2107", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2010-2106", "NVD-CWE-noinfo");
		MAP_CVES_TO_CWES.put("CVE-2010-2105", "NVD-CWE-DesignError");
		MAP_CVES_TO_CWES.put("CVE-2010-1770", "CWE-94");

		/*
		CVE-2009-3548  	Credentials Management (CWE-255)
		CVE-2009-2902 	Path Traversal (CWE-22)
		CVE-2009-2901 	"Permissions, Privileges, and Access Control (CWE-264) "
		CVE-2009-2693 	Path Traversal (CWE-22)
		CVE-2009-0783 (fixed) 	Information Leak / Disclosure (CWE-200)
		CVE-2009-0781 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2009-0580 (fixed) 	Information Leak / Disclosure (CWE-200)
		CVE-2009-0033 (fixed) 	Input Validation (CWE-20)
		
		CVE-2008-5519 	Information Leak / Disclosure (CWE-200)
		CVE-2008-5515 (fixed) 	Path Traversal (CWE-22)
		CVE-2008-4308 (fixed) 	Information Leak / Disclosure (CWE-200)
		CVE-2008-2938 	Path Traversal (CWE-22)
		CVE-2008-2370 (fixed) 	Path Traversal (CWE-22)
		CVE-2008-1947 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2008-1232 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2008-0128 (fixed) 	Configuration (CWE-16)
		CVE-2007-6286 (fixed) 	
		CVE-2007-5461 (fixed) 	Path Traversal (CWE-22)
		CVE-2007-5342 (fixed) 	
		CVE-2007-5333 (fixed) 	
		CVE-2007-3386 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2007-3385 (fixed) 	Information Leak / Disclosure (CWE-200)
		CVE-2007-3382 (fixed) 	Information Leak / Disclosure (CWE-200)
		CVE-2007-2450 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2007-2449 (fixed) 	
		CVE-2007-1858 (fixed) 	
		CVE-2007-1355 (fixed) 	
		CVE-2007-0450 (fixed) 	Path Traversal (CWE-22)
		CVE-2006-7196 (fixed) 	Cross-Site Scripting (XSS) (CWE-79)
		CVE-2006-7195 (fixed) 	
		CVE-2006-3835
		*/
		
		// Tomcat
		MAP_CVES_TO_IDS.put("CVE-2009-3548", 35);
		MAP_CVES_TO_IDS.put("CVE-2009-2902", 36);
		MAP_CVES_TO_IDS.put("CVE-2009-2901", 37);
		MAP_CVES_TO_IDS.put("CVE-2009-2693", 38);
		MAP_CVES_TO_IDS.put("CVE-2009-0783", 39);
		MAP_CVES_TO_IDS.put("CVE-2009-0781", 40);
		MAP_CVES_TO_IDS.put("CVE-2009-0580", 41);
		MAP_CVES_TO_IDS.put("CVE-2009-0033", 42);
		MAP_CVES_TO_IDS.put("CVE-2008-5519", 43);
		MAP_CVES_TO_IDS.put("CVE-2008-5515", 44);
		MAP_CVES_TO_IDS.put("CVE-2008-4308", 45);
		MAP_CVES_TO_IDS.put("CVE-2008-2938", 46);
		MAP_CVES_TO_IDS.put("CVE-2008-2370", 47);
		MAP_CVES_TO_IDS.put("CVE-2008-1947", 48);
		MAP_CVES_TO_IDS.put("CVE-2008-1232", 49);
		MAP_CVES_TO_IDS.put("CVE-2008-0128", 50);
		MAP_CVES_TO_IDS.put("CVE-2007-6286", 51);
		MAP_CVES_TO_IDS.put("CVE-2007-5461", 52);
		MAP_CVES_TO_IDS.put("CVE-2007-5342", 53);
		MAP_CVES_TO_IDS.put("CVE-2007-5333", 54);
		MAP_CVES_TO_IDS.put("CVE-2007-3386", 55);
		MAP_CVES_TO_IDS.put("CVE-2007-3385", 56);
		MAP_CVES_TO_IDS.put("CVE-2007-3382", 57);
		MAP_CVES_TO_IDS.put("CVE-2007-2450", 58);
		MAP_CVES_TO_IDS.put("CVE-2007-2449", 59);
		MAP_CVES_TO_IDS.put("CVE-2007-1858", 60);
		MAP_CVES_TO_IDS.put("CVE-2007-1355", 61);
		MAP_CVES_TO_IDS.put("CVE-2007-0450", 62);
		MAP_CVES_TO_IDS.put("CVE-2006-7197", 20067197);
		MAP_CVES_TO_IDS.put("CVE-2006-7196", 63);
		MAP_CVES_TO_IDS.put("CVE-2006-7195", 64);
		MAP_CVES_TO_IDS.put("CVE-2006-3835", 65);

		MAP_CVES_TO_CWES.put("CVE-2009-3548", "CWE-255");
		MAP_CVES_TO_CWES.put("CVE-2009-2902", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2009-2901", "CWE-264");
		MAP_CVES_TO_CWES.put("CVE-2009-2693", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2009-0783", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2009-0781", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2009-0580", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2009-0033", "CWE-20");
		MAP_CVES_TO_CWES.put("CVE-2008-5519", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2008-5515", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2008-4308", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2008-2938", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2008-2370", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2008-1947", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2008-1232", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2008-0128", "CWE-16");
		MAP_CVES_TO_CWES.put("CVE-2007-6286", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2007-5461", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2007-5342", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2007-5333", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2007-3386", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2007-3385", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2007-3382", "CWE-200");
		MAP_CVES_TO_CWES.put("CVE-2007-2450", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2007-2449", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2007-1858", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2007-1355", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2007-0450", "CWE-22");
		MAP_CVES_TO_CWES.put("CVE-2006-7197", "CWE-119"); // XXX: Notice, CVE-2006-7197 is also CWE-200 
		MAP_CVES_TO_CWES.put("CVE-2006-7196", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2006-7195", "CWE-79");
		MAP_CVES_TO_CWES.put("CVE-2006-3835", ""); // XXX: Unknown; did not look up on NVD and SATE did not provide; it's not trained on as well
	}
	
	
	private ArrayList<FileItem> oReferenceData = null; 
	
	
	/**
	 * Hashes "config string" -&gt; Vector(FirstMatchPoint(XSuccesses, YFailures), SecondMatchPoint(XSuccesses, YFailures)).
	 */
	private Hashtable<String,Vector<Object>> oStatsPerConfig = null;

	/**
	 * Array of sorted stats refs.
	 */
	private transient Vector<Object>[] aoSortedStatsRefs = null;

	/**
	 * A vector of vectors of writers info pre-loaded on <code>connect()</code>.
	 * @see #connect()
	 */
	private Hashtable<Integer,Vector<Object>> oDB = null;

	/**
	 * "Database connection".
	 */
	private IDatabaseConnection oConnection = null;

	/**
	 * For serialization versioning.
	 */
	private static final long serialVersionUID = -5745709358793664345L;

	/**
	 * Constructor.
	 * @param pstrFileName filename of a CSV file with IDs and names of writers
	 */
	public SATEDb(final String pstrFileName)
	throws StorageException
	{
		this.strFilename = pstrFileName;
		this.oDB = new Hashtable<Integer,Vector<Object>>();
		this.oStatsPerConfig = new Hashtable<String,Vector<Object>>();
		this.oReferenceData = new ArrayList<FileItem>();
	}

	@SuppressWarnings("unchecked")
	public final List<Integer> getIDsByFilename(final String pstrFileName, final boolean pbTraining)
	throws StorageException
	{
		Vector<Integer> oIDsToReturn = null;
		
		Debug.debug("getIDByFilename(" + pstrFileName + ", " + pbTraining +")");
		Debug.debug("getIDByFilename: oDB.size(): " + this.oDB.size());
		
		// Traverse all the info vectors looking for sample filename
		for(Integer oID: this.oDB.keySet())
		{
			//Debug.debug("SATEDb: var: " + this.oDB + "");
			//Debug.debug("SATEDb: File: " + strFilenameToLookup + ", id = " + oID.intValue());

			Vector<Object> oSubjectInfo = this.oDB.get(oID);

			Debug.debug("getIDByFilename: oSubjectInfo: " + oSubjectInfo);

			Vector<String> oFilenames = (Vector<String>)oSubjectInfo.elementAt(1);
			
			//Debug.debug("oFilenames: !!!!! File: " + oFilenames);

			if(oFilenames != null)
			{
				// Is our file in any of the categories?
				for(String strCurrentFilename: oFilenames)
				{
					if(strCurrentFilename.equals(pstrFileName))
					{
						if(oIDsToReturn == null)
						{
							oIDsToReturn = new Vector<Integer>();
						}

						// Match found; accumulate category IDs this file belongs to
						oIDsToReturn.add(oID);
						break;
					}
				}
			}

		}

		return oIDsToReturn;
	}
	
	/**
	 * Retrieves subject's ID by a sample filename.
	 * @param pstrFileName Name of a .wav file for which ID must be returned
	 * @param pbTraining indicates whether the filename is a training (<code>true</code>) sample or testing (<code>false</code>)
	 * @return int ID
	 * @throws StorageException in case of an error in any I/O operation
	 */
	// XXX: need to adjust to return multiple IDs per file name, e.g. multiple CVEs 
	public final int getIDByFilename(final String pstrFileName, final boolean pbTraining)
	//public final Vector<Integer> getIDByFilename(final String pstrFileName, final boolean pbTraining)
	throws StorageException
	{
		//Collection<Integer> oCategories = getIDsByFilename(pstrFileName, pbTraining);
		//return oCategories == null ? -1 : oCategories.iterator().next();
		List<Integer> oCategories = getIDsByFilename(pstrFileName, pbTraining);
		return oCategories == null || oCategories.size() == 0 ? -1 : oCategories.get(0);
	}

	/**
	 * Retrieves writer's name by their ID.
	 * @param piID ID of a person in the DB to return a name for
	 * @return name string
	 * @throws StorageException
	 */
	public final String getName(final int piID)
	throws StorageException
	{
		//Debug.debug("getName() - ID = " + piID + ", db size: " + oDB.size());
		String strName;

		Vector<Object> oDBEntry = this.oDB.get(piID);

		if(oDBEntry == null)
		{
			strName = "Unknown Subject (" + piID + ")";
		}
		else
		{
			strName = oDBEntry.elementAt(0).toString();
		}

		return strName;
	}

	/**
	 * Connects to the "database" of writers (opens the file :-)).
	 * @throws StorageException in case of any I/O error
	 */
	public void connect()
	throws StorageException
	{
		// That's where we should establish file linkage and keep it until closed
		//try
		{
			//this.oConnection = new BufferedReader(new FileReader(this.strFilename));
			this.oConnection = GenericMetaXMLLoader.getInstance();
			this.oConnection.initialize(this.strFilename);
			this.bConnected = true;
		}/*
		catch(IOException e)
		{
			throw new StorageException
			(
				"Error opening writer DB: \"" + this.strFilename + "\": " +
				e.getMessage() + "."
			);
		}*/
	}
	
	/**
	 * CWE as a category. 
	 */
	private boolean bCWE = false;
	
	public boolean isCWE()
	{
		return this.bCWE;
	}

	public void setCWE(boolean pbCWE)
	{
		this.bCWE = pbCWE;
	}

	/**
	 * Retrieves writer's data from the text file and populates
	 * internal data structures. Uses StringTokenizer to parse
	 * data read from the file.
	 * @throws StorageException in case of any I/O error
	 */
	public void query()
	throws StorageException
	{
		// That's where we should load db results into internal data structure
		
		Integer iID = -1;

		Map<String,Integer> oCategoryToIDMap = null;
		
		if(this.bCWE == false)
		{
			oCategoryToIDMap = MAP_CVES_TO_IDS;
		}
		else
		{
			oCategoryToIDMap = MAP_CWEIDS_TO_IDS;
		}
		
		// Mapping countries to the lists of filenames
		HashMap<Integer,Vector<String>> oCategoryMapIds = new HashMap<Integer,Vector<String>>(); 

		Debug.enableDebug();

		try
		{
			for(FileItem oRecord: ((GenericMetaXMLLoader)this.oConnection).getFileItems())
			{
				if(oRecord.isLocationless() == true)
				{
					continue;
				}
				
				Debug.debug("SATEDb.query(): processing non-empty record with locations " + oRecord);

				Vector<MetaLocation> oLocations = oRecord.getLocations();
				
				for(MetaLocation oLocation: oLocations)
				{
					if(oLocation.isToIgnore() == true)
					{
						System.out.println("query: The meta location below was indicated to be ignored:");
						System.out.println("===========================================================");
						System.out.println(oLocation);
						System.out.println("===========================================================");
						continue;
					}
					
					//for(String strCVE: oLocation.getCVEs())
					for(Serializable strCXE: this.bCWE ? oLocation.getCWEs() : oLocation.getCVEs())
					{
//						iID = MAP_CVES_TO_IDS.get(strCVE);
						
						String strCategoryKey = "";
						
						if(this.bCWE == false)
						{
							strCategoryKey = strCXE.toString();
						}
						else
						{
							strCategoryKey = ((CWE)strCXE).getCWEID();
						}
						
						iID = oCategoryToIDMap.get(strCategoryKey);

						//Vector<String> oAssociatedFilePathsPerCVE = oCategoryMapIds.get(iID);
						Vector<String> oAssociatedFilePathsPerCategory = oCategoryMapIds.get(iID);
						
						if(oAssociatedFilePathsPerCategory == null)
						{
							oAssociatedFilePathsPerCategory = new Vector<String>();
							oCategoryMapIds.put(iID, oAssociatedFilePathsPerCategory);
						}
						
						oAssociatedFilePathsPerCategory.add(oRecord.getPath());
					}
				}
			}
			
			//for(String strCVE: MAP_CVES_TO_IDS.keySet())
			for(String strCXE: oCategoryToIDMap.keySet())
			{
				// A 2-element vector: "CXE, Vector of filenames"
				Vector<Object> oRecordInfo = new Vector<Object>();
				
				//iID = MAP_CVES_TO_IDS.get(strCVE);
				iID = oCategoryToIDMap.get(strCXE);

				oRecordInfo.add(strCXE);

				if(oCategoryMapIds.get(iID) != null)
				{
					oRecordInfo.add(oCategoryMapIds.get(iID));
				}
				else
				{
					oRecordInfo.add(new Vector<String>());
				}
				
				/*
				if(oCategoryMapIds.get(iID) != null)
				{
					oRecordInfo.add(strCVE);
					oRecordInfo.add(oCategoryMapIds.get(iID));
					
					this.oDB.put(iID, oRecordInfo);
				}
				*/

				this.oDB.put(iID, oRecordInfo);
			}
			
			System.out.println("SATEDb.query(): DB size: " + this.oDB.size());
			Debug.debug("SATEDb.query(): DB: <<<" + this.oDB + ">>>");
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new StorageException(e);
		}
	}

	public List<FileItem> getFileItemsBySubject(String pstrCXE)
	{
		System.out.println("getFileItemsBySubject: " + pstrCXE);
		if(pstrCXE.matches("CVE-.*"))
		{
			return getFileItemsByCVE(pstrCXE);
		}
		else
		{
			return getFileItemsByCWE(pstrCXE);
		}
	}
	
	public List<FileItem> getFileItemsByCVE(String pstrCVE)
	{
		//this.oReferenceData = GenericMetaXMLLoader.getInstance().getFileItems();
		System.out.println("getFileItemsByCVE: " + pstrCVE);
		System.out.println("getFileItemsByCVE: oReferenceData size: " + oReferenceData.size());
		
		ArrayList<FileItem> oItems = new ArrayList<FileItem>();
		
		//for(FileItem oFileItem: GenericMetaXMLLoader.getInstance().getFileItems())
		for(FileItem oFileItem: this.oReferenceData)
		{
			for(MetaLocation oLocation: oFileItem.getLocations())
			{
				if(oLocation.getCVEs().contains(pstrCVE))
				{
					oItems.add(oFileItem);
					break;
				}
			}
		}
		
		return oItems;
	}

	public List<FileItem> getFileItemsByCWE(String pstrCWE)
	{
		System.out.println("getFileItemsByCWE: " + pstrCWE);
		System.out.println("getFileItemsByCWE: oReferenceData size: " + oReferenceData.size());
		
		ArrayList<FileItem> oItems = new ArrayList<FileItem>();
		
		//for(FileItem oFileItem: GenericMetaXMLLoader.getInstance().getFileItems())
		for(FileItem oFileItem: this.oReferenceData)
		{
			for(MetaLocation oLocation: oFileItem.getLocations())
			{
				for(CWE oCWE: oLocation.getCWEs())
				{
					if(oCWE.getCWEID().equals(pstrCWE))
					{
						oItems.add(oFileItem);
						break;
					}
				}
			}
		}
		
		return oItems;
	}

	public List<MetaLocation> getMetaLocationsBySubject(String pstrCXE)
	{
		if(pstrCXE.matches("CVE-.*"))
		{
			return getMetaLocationsByCVE(pstrCXE);
		}
		else
		{
			return getMetaLocationsByCWE(pstrCXE);
		}
	}
	
	public List<MetaLocation> getMetaLocationsByCVE(String pstrCVE)
	{
		return null;
	}

	public List<MetaLocation> getMetaLocationsByCWE(String pstrCWE)
	{
		return null;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value oReferenceData field.
	 */
	public ArrayList<FileItem> getReferenceData()
	{
		return this.oReferenceData;
	}

	/**
	 * Closes (file) database connection.
	 * @throws StorageException if not connected or fails to close inner reader
	 */
	public void close()
	throws StorageException
	{
		// Close file
		if(this.bConnected == false)
		{
			throw new StorageException("SATEDb.close() - not connected");
		}

		//try
		{
			this.oConnection.close();
			this.bConnected = false;
		}
		//catch(IOException e)
		//{
///			throw new StorageException(e.getMessage());
	//	}
	}

	/**
	 * Adds one more classification statics entry.
	 * @param pstrConfig String representation of the configuration the stats are for
	 * @param pbSuccess <code>true</code> if classification was successful; <code>false</code> otherwise
	 */
	public final void addStats(final String pstrConfig, final boolean pbSuccess)
	{
		addStats(pstrConfig, pbSuccess, false);
	}

	/**
	 * Adds one more classification statics entry and accounts for the second best choice.
	 * @param pstrConfig String representation of the configuration the stats are for
	 * @param pbSuccess <code>true</code> if classification was successful; <code>false</code> otherwise
	 * @param pbSecondBest <code>true</code> if classification was successful; <code>false</code> otherwise
	 */
	public final void addStats(final String pstrConfig, final boolean pbSuccess, final boolean pbSecondBest)
	{
		Vector<Object> oMatches = this.oStatsPerConfig.get(pstrConfig);
		Point oPoint = null;

		if(oMatches == null)
		{
			oMatches = new Vector<Object>(3);
			oMatches.add(new Point());
			oMatches.add(new Point());
			oMatches.add(pstrConfig);
		}
		else
		{
			if(pbSecondBest == false)
			{
				// First match
				oPoint = (Point)oMatches.elementAt(0);
			}
			else
			{
				// Second best match
				oPoint = (Point)oMatches.elementAt(1);
			}
		}

		int iSuccesses = 0; // # of successes
		int iFailures = 0; // # of failures

		if(oPoint == null) // Didn't exist yet; create new
		{
			if(pbSuccess == true)
			{
				iSuccesses = 1;
			}
			else
			{
				iFailures = 1;
			}

			oPoint = new Point(iSuccesses, iFailures);

			if(oPoint == null)
			{
				System.err.println("SATEDb.addStats() - oPoint is null! Out of memory?");
				System.exit(-1);
			}

			if(oMatches == null)
			{
				System.err.println("SATEDb.addStats() - oMatches is null! Out of memory?");
				System.exit(-1);
			}

			if(oMatches.size() == 0)
			{
				System.err.println("SATEDb.addStats() - oMatches.size = 0");
				System.exit(-1);
			}

			if(pbSecondBest == false)
			{
				oMatches.setElementAt(oPoint, 0);
			}
			else
			{
				oMatches.setElementAt(oPoint, 1);
			}

			this.oStatsPerConfig.put(pstrConfig, oMatches);
		}

		else // There is an entry for this config; update
		{
			if(pbSuccess == true)
			{
				oPoint.x++;
			}
			else
			{
				oPoint.y++;
			}
		}
	}

	/**
	 * Dumps all collected statistics to STDOUT.
	 * @throws Exception
	 */
	public final void printStats()
	throws Exception
	{
		printStats(false);
	}

	/**
	 * Dumps collected statistics to STDOUT.
	 * @param pbBestOnly <code>true</code> - print out only the best score number; <code>false</code> - all stats
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public final void printStats(boolean pbBestOnly)
	throws Exception
	{
		if(this.oStatsPerConfig.size() == 0)
		{
			System.err.println("SATEDb: no statistics available. Did you run the recognizer yet?");
			return;
		}

		// First row is for the identified results, 2nd is for 2nd best ones.
		String[][] astrResults = new String[2][this.oStatsPerConfig.size()];

		this.aoSortedStatsRefs = oStatsPerConfig.values().toArray(new Vector[0]);
		Arrays.sort(this.aoSortedStatsRefs, new StatsPercentComparator(StatsPercentComparator.DESCENDING));

		int iResultNum = 0;

		System.out.println("guess,run,config,good,bad,%");

		for(int i = 0; i < this.aoSortedStatsRefs.length; i++)
		{
			String strConfig = this.aoSortedStatsRefs[i].elementAt(2).toString();

			for(int j = 0; j < 2; j++)
			{
				Point oGoodBadPoint = (Point)(this.aoSortedStatsRefs[i]).elementAt(j);
				String strGuess = (j == 0) ? "1st" : "2nd";
				String strRun = (iResultNum + 1) + "";
				DecimalFormat oFormat = new DecimalFormat("#,##0.00;(#,##0.00)");
				double dRate = ((double)oGoodBadPoint.x / (double)(oGoodBadPoint.x + oGoodBadPoint.y)) * 100;

				if(pbBestOnly == true)
				{
					System.out.print(oFormat.format(dRate));
					return;
				}

				astrResults[j][iResultNum] =
					strGuess + "," +
					strRun + "," +
					strConfig + "," +
					oGoodBadPoint.x + "," + // Good
					oGoodBadPoint.y + "," + // Bad
					oFormat.format(dRate);
			}

			iResultNum++;
		}

		// Print all of the 1st match
		for(int i = 0; i < astrResults[0].length; i++)
		{
			System.out.println(astrResults[0][i]);
		}

		// Print all of the 2nd match
		for(int i = 0; i < astrResults[1].length; i++)
		{
			System.out.println(astrResults[1][i]);
		}
	}

	/**
	 * Resets in-memory and on-disk statistics.
	 * @throws StorageException
	 */
	public final void resetStats()
	throws StorageException
	{
		this.oStatsPerConfig.clear();
		dump();
	}

	/**
	 * Dumps statistic's Hashtable object as gzipped binary to disk.
	 * @throws StorageException
	 */
	public void dump()
	throws StorageException
	{
		try
		{
			FileOutputStream oFOS = new FileOutputStream(this.strFilename + ".stats");
			GZIPOutputStream oGZOS = new GZIPOutputStream(oFOS);
			ObjectOutputStream oOOS = new ObjectOutputStream(oGZOS);

			oOOS.writeObject(this.oStatsPerConfig);
			oOOS.flush();
			oOOS.close();
			
			//oFOS = new FileOutputStream(this.strFilename + ".ref");
			oFOS = new FileOutputStream(this.oReferenceData.getClass().getName() + ".ref");
			oGZOS = new GZIPOutputStream(oFOS);
			oOOS = new ObjectOutputStream(oGZOS);

			oOOS.writeObject(this.oReferenceData);
			oOOS.flush();
			oOOS.close();
		}
		catch(Exception e)
		{
			throw new StorageException(e);
		}
	}

	/**
	 * Reloads statistic's Hashtable object from disk.
	 * If the file did not exist, it creates a new one.
	 * @throws StorageException
	 */
	@SuppressWarnings("unchecked")
	public void restore()
	throws StorageException
	{
		try
		{
			FileInputStream oFIS = new FileInputStream(this.strFilename + ".stats");
			GZIPInputStream oGZIS = new GZIPInputStream(oFIS);
			ObjectInputStream oOIS = new ObjectInputStream(oGZIS);

			this.oStatsPerConfig = (Hashtable<String, Vector<Object>>)oOIS.readObject();
			oOIS.close();
		}
		catch(FileNotFoundException e)
		{
			System.out.println
			(
				"NOTICE: File " + this.strFilename +
				".stats does not seem to exist. Creating a new one...."
			);

			resetStats();
		}
		catch(ClassNotFoundException e)
		{
			throw new StorageException
			(
				"SATEDb.retore() - ClassNotFoundException: " +
				e.getMessage()
			);
		}
		catch(Exception e)
		{
			throw new StorageException(e);
		}
		
		try
		{
			//FileInputStream oFIS = new FileInputStream(this.strFilename + ".ref");
			FileInputStream oFIS = new FileInputStream(this.oReferenceData.getClass().getName() + ".ref");
			GZIPInputStream oGZIS = new GZIPInputStream(oFIS);
			ObjectInputStream oOIS = new ObjectInputStream(oGZIS);

			this.oReferenceData = (ArrayList<FileItem>)oOIS.readObject();
			oOIS.close();
		}
		catch(FileNotFoundException e)
		{
			System.out.println
			(
				"NOTICE: File " + this.strFilename +
				".ref does not seem to exist. Creating a new one...."
			);

			dump();
		}
		catch(ClassNotFoundException e)
		{
			throw new StorageException
			(
				"SATEDb.retore() - ClassNotFoundException: " +
				e.getMessage()
			);
		}
		catch(Exception e)
		{
			throw new StorageException(e);
		}
	}

	public void syncReferenceData()
	throws StorageException
	{
		this.oReferenceData = ((GenericMetaXMLLoader)this.oConnection).getFileItems();
		dump();
	}
	
	/**
	 * Allows querying for TODO.
	 * @return returns the value oConnection field.
	 */
	public IDatabaseConnection getConnection()
	{
		return this.oConnection;
	}
}

/**
 * <p>Used in sorting by percentage of the stats entries
 * in either ascending or descending order.</p>
 *
 * <p>TODO: To be moved to Stats.</p>
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.14 $
 * @since 0.0.1 of MARF
 */
class StatsPercentComparator
extends marf.util.SortComparator
{
	/**
	 * For serialization versioning.
	 * @since 0.3.0.5
	 */
	private static final long serialVersionUID = -7185805363856188810L;

	/**
	 * Mimics parent's constructor.
	 */
	public StatsPercentComparator()
	{
		super();
	}

	/**
	 * Mimics parent's constructor.
	 * @param piSortMode either DESCENDING or ASCENDING sort mode
	 */
	public StatsPercentComparator(final int piSortMode)
	{
		super(piSortMode);
	}

	/**
	 * Implementation of the Comparator interface for the stats objects.
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public int compare(Object poStats1, Object poStats2)
	{
		Vector<Object> oStats1 = (Vector<Object>)poStats1;
		Vector<Object> oStats2 = (Vector<Object>)poStats2;

		Point oGoodBadPoint1 = (Point)(oStats1.elementAt(0));
		Point oGoodBadPoint2 = (Point)(oStats2.elementAt(0));

		double dRate1 = ((double)oGoodBadPoint1.x / (double)(oGoodBadPoint1.x + oGoodBadPoint1.y)) * 100;
		double dRate2 = ((double)oGoodBadPoint2.x / (double)(oGoodBadPoint2.x + oGoodBadPoint2.y)) * 100;

		switch(this.iSortMode)
		{
			case DESCENDING:
			{
				return (int)((dRate2 - dRate1) * 100);
			}

			case ASCENDING:
			default:
			{
				return (int)((dRate1 - dRate2) * 100);
			}
		}
	}
}

// EOF
