package SATE;

import java.util.Vector;


/**
 * TODO: document.
 *
 * @author serguei
 * @since Jul 30, 2010
 * @version $Id: Warning.java,v 1.6 2010/09/12 01:55:27 mokhov Exp $
 */
public class Warning
{
	public static final Integer GRADE_UNKNOWN_SEVERITY = 2;
	public static final Double GRADE_UNKNOWN_PROBABILITY = 0.0;
	public static final Double GRADE_UNKNOWN_TOOL_SPECIFIC_METRIC = 0.0;
	
/*	
    * Id - a simple counter, unique within SATE 2010.
    * (Optional) tool specific id.
    * One or more locations, where each location has:
          o (Optional) id - path id. If a tool produces several paths for a weakness, id can be used to differentiate between them.
          o line - line number.
          o path - file path.
          o (Optional) fragment - a relevant source code fragment at the location.
          o (Optional) explanation - why the location is relevant or what variable is affected.
    * Name (class) of the weakness, e.g., "buffer overflow".
    * (Optional) CWE id, where applicable.
    * Weakness grade (assigned by the tool):
          o Severity on the scale 1 to 5, with 1 - the highest.
          o (Optional) probability that the problem is a true positive, from 0 to 1.
          o (Optional) tool_specific_rank - tool specific metric - useful if a tool does not use severity and probability. If a team uses this field, it would have to separately provide definition, scale, and possible values. 
    * Output - original message from the tool about the weakness, either in plain text, HTML, or XML.
    * (Optional) An evaluation of the warning by a human; not considered to be part of tool output, including:
          o (Optional) correctness - human analysis of the weakness, one of several categories. Use this instead of the deprecated "falsepositive" attribute
*/
	public static String escapeForXMLOut(String pstrRawString)
	{
		String strEscapedString = pstrRawString.replaceAll("&", "&amp;");
		strEscapedString = strEscapedString.replaceAll("<", "&lt;");
		strEscapedString = strEscapedString.replaceAll(">", "&gt;");
		
		return strEscapedString;
	}
	
	protected Integer iSATEID = -1;
	protected Integer iMARFID = -1;
	protected Vector<Location> oLocations = new Vector<Location>();
	
	public class Location
	{
		String strPathID = "0";
		Integer iLineNo = 0;
		String strFilePath = "/unknown";
		String strCodeFragment = "unknown();";
		String strExplanation = "";

		public Location()
		{
			
		}
		
		/**
		 * @param piLineNo
		 * @param pstrFilePath
		 * @param pstrCodeFragment
		 * @param pstrExplanation
		 * @param strPatphID
		 */
		public Location(Integer piLineNo, String pstrFilePath, String pstrCodeFragment, String pstrExplanation, String strPatphID)
		{
			super();
			this.iLineNo = piLineNo;
			this.strFilePath = pstrFilePath;
			this.strCodeFragment = pstrCodeFragment;
			this.strExplanation = pstrExplanation;
			this.strPathID = strPatphID;
		}

		public Location(Integer piLineNo, String pstrFilePath)
		{
			super();
			this.iLineNo = piLineNo;
			this.strFilePath = pstrFilePath;
		}

		public Location(Integer piLineNo, String pstrFilePath, String pstrCodeFragment, String pstrExplanation)
		{
			super();
			this.iLineNo = piLineNo;
			this.strFilePath = pstrFilePath;
			this.strCodeFragment = pstrCodeFragment;
			this.strExplanation = pstrExplanation;
		}

		
		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString()
		{
			/*
		<location id="234" line="0" path="/foo/bar">
			<fragment>
	foo bar
	printf();
	//testing testing
			</fragment>
			<explanation>
	Self-explanatory
			</explanation>
		</location>

			 */
			
			StringBuffer strXMLOut = new StringBuffer(); 
			
			strXMLOut
				.append("\t\t<location id=\"")
					.append(strPathID)
					.append("\" line=\"").append(iLineNo)
					.append("\" path=\"").append(strFilePath).append("\">\n")
				.append("\t\t\t<fragment>\n").append(escapeForXMLOut(strCodeFragment)).append("\n\t\t\t</fragment>\n")
				.append("\t\t\t<explanation>\n").append(escapeForXMLOut(strExplanation)).append("\n\t\t\t</explanation>\n")
				.append("\t\t</location>\n")
			;
			
			return strXMLOut.toString();
		}
	}
	
	protected String strName = "Unknown";
	protected Integer iCWE = 0;

	protected WeaknessGrade oWeaknessGrade = new WeaknessGrade();
	
	protected String strOutput = "";
	protected String strCorrectness = "";
	
	
	class WeaknessGrade
	{
		Integer iSeverity = -1;
		Double dProbability = -1.0;
		Double dToolSpecificMetric = 0.0;

		public WeaknessGrade()
		{
			
		}
		
		public WeaknessGrade(Integer piSeverity, Double pdProbability, Double pdToolSpecificMetric)
		{
			super();
			iSeverity = piSeverity;
			dProbability = pdProbability;
			dToolSpecificMetric = pdToolSpecificMetric;
		}


		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString()
		{
			/*
			<grade
				severity="1"
				probability=".2"
				tool_specific_rank="1.454545454"
			/>
			 */
			
			StringBuffer strXMLOut = new StringBuffer(); 
			
			strXMLOut
				.append("\t\t<grade\n")
					.append("\t\t\tseverity=\"").append(iSeverity).append("\"\n")
					.append("\t\t\tprobability=\"").append(dProbability).append("\"\n")
					.append("\t\t\ttool_specific_rank=\"").append(dToolSpecificMetric).append("\"\n")
				.append("\t\t/>\n")
			;
			
			return strXMLOut.toString();
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Object clone()
	throws CloneNotSupportedException
	{
		return super.clone();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		// TODO Auto-generated method stub
		return super.equals(obj);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode()
	{
		// TODO Auto-generated method stub
		return super.hashCode();
	}

	public void addLocation(Integer piLineNo, String pstrFilePath, String pstrCodeFragment, String pstrExplanation, String pstrPathID)
	{
		Location oLocation = new Location(piLineNo, pstrFilePath, pstrCodeFragment, pstrExplanation, pstrPathID);
		this.oLocations.add(oLocation);
	}

	public void addLocation(Integer piLineNo, String pstrFilePath)
	{
		Location oLocation = new Location(piLineNo, pstrFilePath);
		this.oLocations.add(oLocation);
	}

	public void setWeaknessGrade(Double pdToolSpecificMetric)
	{
		setWeaknessGrade(GRADE_UNKNOWN_SEVERITY, GRADE_UNKNOWN_PROBABILITY, pdToolSpecificMetric);
	}

	public void setWeaknessGrade(Double pdProbability, Double pdToolSpecificMetric)
	{
		setWeaknessGrade(GRADE_UNKNOWN_SEVERITY, pdProbability, pdToolSpecificMetric);
	}

	public void setWeaknessGrade(Integer piSeverity)
	{
		setWeaknessGrade(piSeverity, GRADE_UNKNOWN_PROBABILITY, GRADE_UNKNOWN_TOOL_SPECIFIC_METRIC);
	}
	
	public void setWeaknessGrade(Integer piSeverity, Double pdProbability, Double pdToolSpecificMetric)
	{
		this.oWeaknessGrade = new WeaknessGrade(piSeverity, pdProbability, pdToolSpecificMetric);
	}
	
	/**
	 * Allows querying for TODO.
	 * @return returns the value iSATEID field.
	 */
	public Integer getSATEID()
	{
		return iSATEID;
	}

	/**
	 * Allows setting TODO.
	 * @param isateid the new value of iSATEID to set.
	 */
	public void setSATEID(Integer isateid)
	{
		iSATEID = isateid;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value iMARFID field.
	 */
	public Integer getMARFID()
	{
		return iMARFID;
	}

	/**
	 * Allows setting TODO.
	 * @param imarfid the new value of iMARFID to set.
	 */
	public void setMARFID(Integer imarfid)
	{
		iMARFID = imarfid;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value strName field.
	 */
	public String getName()
	{
		return strName;
	}

	/**
	 * Allows setting TODO.
	 * @param strName the new value of strName to set.
	 */
	public void setName(String strName)
	{
		this.strName = strName;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value iCWE field.
	 */
	public Integer getCWE()
	{
		return iCWE;
	}

	/**
	 * Allows setting TODO.
	 * @param icwe the new value of iCWE to set.
	 */
	public void setCWE(Integer icwe)
	{
		iCWE = icwe;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value strOutput field.
	 */
	public String getOutput()
	{
		return strOutput;
	}

	/**
	 * Allows setting TODO.
	 * @param strOutput the new value of strOutput to set.
	 */
	public void setOutput(String strOutput)
	{
		this.strOutput = strOutput;
	}

	/**
	 * Allows querying for TODO.
	 * @return returns the value strCorrectness field.
	 */
	public String getCorrectness()
	{
		return strCorrectness;
	}

	/**
	 * Allows setting TODO.
	 * @param strCorrectness the new value of strCorrectness to set.
	 */
	public void setCorrectness(String strCorrectness)
	{
		this.strCorrectness = strCorrectness;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		StringBuffer strXMLOut = new StringBuffer();
		
		strXMLOut
		.append("\t<weakness id=\"")
		.append(iSATEID)
		.append("\" tool_specific_id=\"").append(iMARFID).append("\">\n")
			.append("\t\t<name cweid=\"").append(this.iCWE).append("\">")
			.append(strName)
			.append("</name>\n");

		for(Location oLocation: this.oLocations)
		{
			strXMLOut.append(oLocation);
		}
			
		strXMLOut
			.append(this.oWeaknessGrade)
			
			.append("\t\t<output>\n")
			.append("\t\t\t<textoutput>\n")
			.append("\t\t\t").append(escapeForXMLOut(this.strOutput)).append("\n")
			.append("\t\t\t</textoutput>\n")
			.append("\t\t</output>\n")
	
			.append("\t\t<!--\n")
			.append("\t\t\tHuman evaluation.\n")
			.append("\t\t\tseverity:    1 ... 5\n")
			.append("\t\t\tcorrectness: true, quality, insignificant, unknown, false\n")
			.append("\t\t-->\n")
			.append("\t\t<!--evaluation severity=\"1\" correctness=\"true\">\n")
			.append("\t\t\t<comments>\n")
			.append("\t\t\tHuman evaluation comments.\n")
			.append("\t\t\t</comments>\n")
			.append("\t\t</evaluation-->\n")
		.append("\t</weakness>\n")
		;
		
		return strXMLOut.toString();
	}
}

// EOF
