package Storage;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import javax.sound.sampled.AudioInputStream;

import marf.Storage.Sample;
import marf.Storage.StorageException;
import marf.Storage.Loaders.WAVLoader;
import marf.util.Debug;
import marf.util.InvalidSampleFormatException;


/**
 * @author serguei
 *
 */
public class AnyToWAVLoader
extends WAVLoader
{

	/**
	 * @throws InvalidSampleFormatException
	 */
	public AnyToWAVLoader()
	throws InvalidSampleFormatException
	{
		super();
		Debug.debug("AnyToWAVLoader constructed.");
	}

	/**
	 * Override to wrap any binary file data into a properly
	 * formatted AudioInputStream to fool the rest of the sample
	 * loader.
	 *
	 * @see marf.Storage.SampleLoader#loadSample(java.io.File)
	 */
	public Sample loadSample(File poInFile)
	throws StorageException
	{
		try
		{
			if(poInFile != null && poInFile.isFile())
			{
				return loadSample
				(
					new AudioInputStream
					(
						new BufferedInputStream(new FileInputStream(poInFile)),
						this.oAudioFormat,
						poInFile.length() / this.oAudioFormat.getFrameSize()
					)
				);
			}
			else
			{
				throw new FileNotFoundException("Filename is either null or is not a regular file.");
			}
		}

		// To avoid re-wrapping into StorageException again.
		catch(StorageException e)
		{
			e.printStackTrace(System.err);
			throw e;
		}

		// Wrap all the other exceptions here.
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new StorageException(e);
		}
	}
}

// EOF
