package Storage;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import marf.Configuration;
import marf.Storage.IDatabaseConnection;
import marf.Storage.StorageException;
import marf.util.Debug;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


/**
 * <p>XML.</p>
 *
 * @author Serguei Mokhov
 *
 * @version $Id: GenericMetaXMLLoader.java,v 1.15 2010/09/06 23:41:59 mokhov Exp $
 * @since 0.0.1
 */
public class GenericMetaXMLLoader
implements IDatabaseConnection
{
	// Should be a singleton pool?
	// Should be appendable from multiple files?

	/*
	 * ----------------------
	 * Data Members
	 * ----------------------
	 */

	/**
	 * Collection of layers.
	 */
	private ArrayList<FileItem> oFileItems = new ArrayList<FileItem>();

	/**
	 * Current layer.
	 */
	private transient FileItem oCurrentFileItem = new FileItem();


	private ArrayList<String> oFilenamesLoaded = new ArrayList<String>();
	
	/* Constants used for JAXP 1.2 */

	/**
	 * All output will use this encoding.
	 */
	public static final String OUTPUT_ENCODING = "UTF-8";

	/**
	 * JAXP 1.2 Schema.
	 */
	public static final String JAXP_SCHEMA_LANGUAGE =
		"http://java.sun.com/xml/jaxp/properties/schemaLanguage";

	/**
	 * XML 2001 Schema.
	 */
	public static final String W3C_XML_SCHEMA =
		"http://www.w3.org/2001/XMLSchema";

	/**
	 * JAXP 1.2 Schema URL.
	 */
	public static final String JAXP_SCHEMA_SOURCE =
		"http://java.sun.com/xml/jaxp/properties/schemaSource";

	/**
	 * For serialization versioning.
	 * When adding new members or make other structural
	 * changes regenerate this number with the
	 * <code>serialver</code> tool that comes with JDK.
	 */
	//private static final long serialVersionUID = 6116721242820120028L;


	/*
	 * ----------------------
	 * Methods
	 * ----------------------
	 */

	/**
	 * NeuralNetwork Constructor.
	 * @param poFeatureExtraction FeatureExtraction module reference
	 */
	private GenericMetaXMLLoader()
	{
	}

	private static GenericMetaXMLLoader soInstance = null;
	
	public static final GenericMetaXMLLoader getInstance()
	{
		if(soInstance == null)
		{
			soInstance = new GenericMetaXMLLoader();
		}
		
		return soInstance;
	}

	/* (non-Javadoc)
	 * @see marf.Storage.IDatabaseConnection#initialize(java.lang.String)
	 */
	public final void initialize(final String pstrFilename)
	throws StorageException
	{
		initialize(pstrFilename, false);
	}

	/**
	 * Parses XML and produces a data structure.
	 * @param pstrFilename net's XML filename
	 * @param pbValidateDTD if true DTD will be validated
	 * @throws StorageException if there was an I/O or otherwise error
	 * during initialization of the net
	 */
	public final void initialize(final String pstrFilename, final boolean pbValidateDTD)
	throws StorageException
	//throws Exception
	{
		// Do nothing if the same filename is fed to us again.
		// The check is shallow and verbatim. No intelligence present
		// if the same file is loaded from different paths.
		if(this.oFilenamesLoaded.contains(pstrFilename))
		{
			return;
		}
		
		this.oFilenamesLoaded.add(pstrFilename);
		
		try
		{
			Debug.debug("Initializing XML parser...");
	
			DocumentBuilderFactory oDBF = DocumentBuilderFactory.newInstance();
	
			oDBF.setNamespaceAware(true);
			oDBF.setValidating(pbValidateDTD);
	
			DocumentBuilder oBuilder = oDBF.newDocumentBuilder();
			
			OutputStreamWriter oErrorWriter = new OutputStreamWriter(System.err, OUTPUT_ENCODING);
	
			oBuilder.setErrorHandler(new MetaXMLErrorHandler(new PrintWriter(oErrorWriter, true)));
	
			Debug.debug("Parsing XML file...");
			Document oDocument = oBuilder.parse(new File(pstrFilename));

			// Build NNet structure
			Debug.debug("Reading XML structure...");
			readXMLDocument(oDocument);

			// Account for the last item in the list
			if(this.oCurrentFileItem.isEmpty() == false)
			{
				if(this.oCurrentCWE.isEmpty() == false)
				{
					this.oCWEs.add(this.oCurrentCWE);
				}
				
				if(this.strCurrentCVE.length() > 0)
				{
					this.oCVEs.add(this.strCurrentCVE);
				}
				
				if(this.oCVEs.size() > 0)
				{
					this.oCurrentLocation.addCVEs(this.oCVEs);
				}
				
				if(this.oCWEs.size() > 0)
				{
					this.oCurrentLocation.addCWEs(this.oCWEs);
				}
				
				if(this.oCurrentLocation.isEmpty() == false)
				{
					this.oLocations.add(this.oCurrentLocation);
				}
				
				if(this.oLocations.size() > 0)
				{
					this.oCurrentFileItem.addLocations(this.oLocations);
				}
				
				this.oFileItems.add(this.oCurrentFileItem);
			}
			
			// Final reset
			this.oCurrentFileItem = new FileItem();
			this.oCurrentLocation = new MetaLocation();
			this.oCurrentCWE = new CWE();
			this.strCurrentCVE = "";
			this.oCVEs = new Vector<String>();
			this.oCWEs = new Vector<CWE>();
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new StorageException(e);
		}
    }

	
	public static final String NODE_CORPUS = "dataset";
	public static final String NODE_DESCRIPTION = "description";
	
	public static final String NODE_FILE = "file";
	public static final String ATTR_FILE_ID = "id";
	public static final String ATTR_FILE_PATH = "path";
	public static final String ATTR_FILE_IGNORE = "ignore";

	public static final String NODE_META = "meta";

	public static final String NODE_FILE_TYPE = "type";
	
	public static final String NODE_LENGTH = "length";
	public static final String ATTR_LENGTH_LINES = "lines";
	public static final String ATTR_LENGTH_WORDS = "words";
	public static final String ATTR_LENGTH_BYTES = "bytes";
	
	public static final String NODE_LOCATION = "location";
	public static final String ATTR_LOCATION_LINE = "line";
	public static final String ATTR_LOCATION_FRAGLINES = "fraglines";
	public static final String ATTR_LOCATION_IGNORE = "ignore";
	public static final String ATTR_LOCATION_TYPE = "type";
	
	public static final String NODE_CVE = "cve";
	
	public static final String NODE_NAME = "name";
	public static final String ATTR_NAME_CWEID = "cweid";
	
	public static final String NODE_FRAGMENT = "fragment";
	public static final String NODE_EXPLANATION = "explanation";
	
	Vector<MetaLocation> oLocations = new Vector<MetaLocation>();
	Vector<String> oCVEs = new Vector<String>();
	Vector<CWE> oCWEs = new Vector<CWE>();
	
	MetaLocation oCurrentLocation = new MetaLocation();
	String strCurrentCVE = "";
	CWE oCurrentCWE = new CWE();
	
	/**
	 * Performs DOM tree traversal to build neural network structure.
	 * @param poNode current root Node
	 */
	private final void readXMLDocument(Node poNode)
	{
		String strNodeName = "";
		
		int iType = poNode.getNodeType();

		switch(iType)
		{
			case Node.ELEMENT_NODE:
			{
				strNodeName = poNode.getNodeName();

				// Ignore the whole description tag interferes the below.
				if(strNodeName.equals(NODE_DESCRIPTION))
				{
					//poNode = poNode.getNextSibling();
				}
				
				/*
				 * <file>
				 */
				else if(strNodeName.equals(NODE_FILE))
				{
					// We reached another <file>...
					// Complete current file item if it's not empty
					//System.err.println("Encountered <file>...");
					//System.err.println("Current FileItem: " + this.oCurrentFileItem);
					//System.err.println("Current Locations: " + this.oLocations);
					
					if(this.oCurrentFileItem.isEmpty() == false)
					{
						/*if(this.strCurrentCVE.length() > 0)
						{
							this.oCVEs.add(this.strCurrentCVE);
						}*/
						
						if(this.oCurrentCWE.isEmpty() == false)
						{
							this.oCWEs.add(this.oCurrentCWE);
						}
						
						if(this.oCVEs.size() > 0)
						{
							this.oCurrentLocation.addCVEs(this.oCVEs);
						}

						if(this.oCWEs.size() > 0)
						{
							this.oCurrentLocation.addCWEs(this.oCWEs);
						}
						
						if(this.oCurrentLocation.isEmpty() == false)
						{
							this.oLocations.add(this.oCurrentLocation);
						}
						
						if(this.oLocations.size() > 0)
						{
							this.oCurrentFileItem.addLocations(this.oLocations);
						}
						
						//System.err.println("FileItem: " + this.oCurrentFileItem);
						this.oFileItems.add(this.oCurrentFileItem);

						// Reset all our items for the new file
						this.oCurrentFileItem = new FileItem();

						this.oLocations = new Vector<MetaLocation>();
						
						this.oCWEs = new Vector<CWE>();
						this.oCVEs = new Vector<String>();
						
						this.oCurrentLocation = new MetaLocation();
						this.strCurrentCVE = "";
						this.oCurrentCWE = new CWE();
					}
					
					// Extract 'id', 'path', 'ignore' attributes
					NamedNodeMap oAtts = poNode.getAttributes();

					for(int i = 0; i < oAtts.getLength(); i++)
					{
						Node oAttribute = oAtts.item(i);

						String strAttName = oAttribute.getNodeName();
						String strAttValue = oAttribute.getNodeValue().trim();
						
						if(strAttName.equals(ATTR_FILE_ID))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setFileID(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_FILE_PATH))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setPath(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_FILE_IGNORE))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setIgnore(strAttValue);
							}
						}
						else
						{
							System.err.println
							(
								"Unhandled file attribute: " + strAttName
								+ " with value " + strAttValue
							);
						}
					}
				}
				
				/*
				 * File type
				 */
				else if(strNodeName.equals(NODE_FILE_TYPE))
				{
					String strLength = poNode.getTextContent().trim();
					
					if(strLength.length() > 0)
					{
						this.oCurrentFileItem.setFileType(strLength);
					}
				}
				
				/*
				 * Lengths
				 */
				else if(strNodeName.equals(NODE_LENGTH))
				{
					// Extract 'lines', 'words', 'bytes' attributes
					NamedNodeMap oAtts = poNode.getAttributes();

					for(int i = 0; i < oAtts.getLength(); i++)
					{
						Node oAttribute = oAtts.item(i);

						String strAttName = oAttribute.getNodeName();
						String strAttValue = oAttribute.getNodeValue().trim();
						
						if(strAttName.equals(ATTR_LENGTH_LINES))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setLines(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_LENGTH_WORDS))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setWords(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_LENGTH_BYTES))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentFileItem.setBytes(strAttValue);
							}
						}
						else
						{
							System.err.println
							(
								"Unhandled length attribute: " + strAttName
								+ " with value " + strAttValue
							);
						}
					}
				}
				
				/*
				 * Location
				 */
				else if(strNodeName.equals(NODE_LOCATION))
				{
					// We reached another <location>...
					// Complete current location if it's not empty
					if(this.oCurrentLocation.isEmpty() == false)
					{
						this.oCWEs.add(this.oCurrentCWE);
						this.oCurrentLocation.addCWEs(this.oCWEs);
						this.oCurrentLocation.addCVEs(this.oCVEs);
						
						//System.err.println("Location: " + oCurrentLocation);
						this.oLocations.add(this.oCurrentLocation);

						// Reset all our items for the new file
						this.oCurrentLocation = new MetaLocation();
						this.oCurrentCWE = new CWE();
						this.strCurrentCVE = "";
						
						this.oCWEs = new Vector<CWE>();
						this.oCVEs = new Vector<String>();
					}
					
					// Extract line number
					NamedNodeMap oAtts = poNode.getAttributes();

					for(int i = 0; i < oAtts.getLength(); i++)
					{
						Node oAttribute = oAtts.item(i);

						String strAttName = oAttribute.getNodeName();
						String strAttValue = oAttribute.getNodeValue().trim();
						
						if(strAttName.equals(ATTR_LOCATION_LINE))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentLocation.setLineNumber(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_LOCATION_FRAGLINES))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentLocation.setFragmentLines(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_LOCATION_IGNORE))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentLocation.setIgnore(strAttValue);
							}
						}
						else if(strAttName.equals(ATTR_LOCATION_TYPE))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentLocation.setType(strAttValue);
							}
						}
						else
						{
							System.err.println
							(
								"Unhandled article attribute: " + strAttName
								+ " with value " + strAttValue
							);
						}
					}
				}

				/*
				 * CVE
				 */
				else if(strNodeName.equals(NODE_CVE))
				{
					this.strCurrentCVE = poNode.getTextContent().trim();
					//System.out.println("strCurrentCVE: " + strCurrentCVE);

					if(this.strCurrentCVE.length() > 0)
					{
						this.oCVEs.add(this.strCurrentCVE);
						//System.out.println("CVEs: " + oCVEs);
					}
				}

				/*
				 * CWE
				 */
				else if(strNodeName.equals(NODE_NAME))
				{
					// We reached another <name>...
					// Complete current CWE if it's not empty
					if(this.oCurrentCWE.isEmpty() == false)
					{
						this.oCWEs.add(this.oCurrentCWE);

//						System.out.println("CWEs: " + oCWEs);

						// Reset all our items for the new file
						this.oCurrentCWE = new CWE();
					}
					
					// Get weakness name
					String strCurrentCWE = poNode.getTextContent().trim();
					
					if(strCurrentCWE.length() > 0)
					{
//						System.out.println("###: strCurrentCWE: " + strCurrentCWE);
						this.oCurrentCWE.setName(strCurrentCWE);
					}

					// Extract CWEID
					NamedNodeMap oAtts = poNode.getAttributes();

					for(int i = 0; i < oAtts.getLength(); i++)
					{
						Node oAttribute = oAtts.item(i);

						String strAttName = oAttribute.getNodeName();
						String strAttValue = oAttribute.getNodeValue().trim();
						
						if(strAttName.equals(ATTR_NAME_CWEID))
						{
							if(strAttValue.length() > 0)
							{
								this.oCurrentCWE.setCWEID(strAttValue);
							}
						}
						else
						{
							System.err.println
							(
								"Unhandled article attribute: " + strAttName
								+ " with value " + strAttValue
							);
						}
					}
					
//					System.out.println("###: oCurrentCWE: " + oCurrentCWE);
				}

				/*
				 * Code fragment
				 */
				else if(strNodeName.equals(NODE_FRAGMENT))
				{
					String strCurrentFragment = poNode.getTextContent().trim();
//					System.out.println("###: Fragment: " + strCurrentFragment + "###");

					if(strCurrentFragment.length() > 0)
					{
						this.oCurrentLocation.setCodeFragment(strCurrentFragment);
					}
				}

				/*
				 * Explanation of the weakness (e.g. taken from NVD, CVE, etc.)
				 */
				else if(strNodeName.equals(NODE_EXPLANATION))
				{
					String strCurrentExplanation = poNode.getTextContent().trim();
//					System.out.println("###: Explanation: " + strCurrentExplanation + "###");

					if(strCurrentExplanation.length() > 0)
					{
						this.oCurrentLocation.setExplanation(strCurrentExplanation);
					}
				}
				
				/*
				 * Something unhandled 
				 */
				else
				{
					//System.err.println("!!!Ignored named node: " + strNodeName);
					//System.err.println("!!!NODE: " + poNode + " encountered and ignored.");
					//System.err.println("!!!NODE text: " + poNode.getTextContent() + " encountered and ignored.");
				}
					
				break;
			}
			
			default:
			{
				//System.err.println("Unhandled NODE TYPE: ?[" + iType + "]? encountered and ignored.");
				//System.err.println("Unhandled NODE TYPE: ?[" + poNode + "]? encountered and ignored.");
				//System.err.println("Unhandled NODE TYPE: ?[" + poNode.getTextContent() + "]? encountered and ignored.");
			}
		}

		// Recurse for children if any
		for
		(
			Node oChild = poNode.getFirstChild();
//			Node oChild = poNode.getNextSibling();
			oChild != null;
			oChild = oChild.getNextSibling()
		)
		{
			readXMLDocument(oChild);
		}
	}

	/**
	 * Indents the output according to the requested tabulation
	 * for pretty indentation. TODO: move elsewhere to some 
	 * utility module.
	 * @param poWriter Writer object to write tabs to.
	 * @param piTabsNum number of tabs
	 * @throws IOException if there is an error writing out the tabs
	 */
	public static final void indent(BufferedWriter poWriter, final int piTabsNum)
	throws IOException
	{
		for(int i = 0; i < piTabsNum; i++)
		{
			poWriter.write("\t");
		}
	}

	/**
	 * O(n).
	 * @param pstrFileID
	 * @return
	 */
	public FileItem lookupFileItemById(String pstrFileID)
	{
		FileItem oResult = null;
		
		if(this.oFileItems.isEmpty() == false)
		{
			for(FileItem oItem: this.oFileItems)
			{
				if(oItem.getFileID().equals(pstrFileID))
				{
					oResult = oItem;
					break;
				}
			}
		}
		
		return oResult;
	}
	
	
	/**
	 * Error handler to report errors and warnings.
	 * TODO: iclement: this may need revision.
	 * @author Ian Clement
	 */
	private static class MetaXMLErrorHandler
	implements ErrorHandler
	{
		/**
		 * Error handler output goes here.
		 */
		private PrintWriter oOut;

		/**
		 * Constructs our error handler with the given writer.
		 * @param poOut writer to write errors to
		 */
		MetaXMLErrorHandler(PrintWriter poOut)
		{
			this.oOut = poOut;
		}

		/**
		 * Returns a string describing parse exception details.
		 * @param poParseException exception to get info from
		 * @return string representation of the info
		 */
		private String getParseExceptionInfo(SAXParseException poParseException)
		{
			String strSystemId = poParseException.getSystemId();

			if(strSystemId == null)
			{
				strSystemId = "null";
			}

			String strErrInfo =
				"URI=" + strSystemId +
				" Line=" + poParseException.getLineNumber() +
				": " + poParseException.getMessage();

			return strErrInfo;
		}

		// The following methods are standard SAX ErrorHandler methods.
		// See SAX documentation for more info.

		/**
		 * Issues a SAX warning.
		 * @param poParseException the parse exception to warn about
		 * @throws SAXException
		 */
		public void warning(SAXParseException poParseException)
		throws SAXException
		{
			this.oOut.println
			(
				"WARNING: "
				+ getParseExceptionInfo(poParseException)
			);
		}

		/**
		 * Issues a SAX error.
		 * @param poParseException the source of the error information
		 * @throws SAXException the error based on the parse exception
		 */
		public void error(SAXParseException poParseException)
		throws SAXException
		{
			String strMessage = "ERROR: " + getParseExceptionInfo(poParseException);
			throw new SAXException(strMessage);
		}

		/**
		 * Issues a SAX fatal error.
		 * @param poParseException the source of the error information
		 * @throws SAXException the error based on the parse exception
		 */
		public void fatalError(SAXParseException poParseException)
		throws SAXException
		{
			String strMessage = "FATAL: " + getParseExceptionInfo(poParseException);
			throw new SAXException(strMessage);
		}
	}
	
	/**
	 * Get the collection of loaded items.
	 * @return
	 */
	public ArrayList<FileItem> getFileItems()
	{
		return this.oFileItems;
	}
	
	@Override
	public void close()
	throws StorageException
	{
		// TODO Auto-generated method stub
	}

	@Override
	public void open()
	throws StorageException
	{
		// TODO Auto-generated method stub
	}

	@Override
	public String readLine()
	throws StorageException
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setConfiguration(Configuration poConfiguration)
	{
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return this.oFileItems.get(0).toString();
	}
}

// EOF
