package Storage;

import java.io.InputStream;

import marf.Storage.ByteArrayFileReader;
import marf.Storage.MARFAudioFileFormat;
import marf.Storage.Sample;
import marf.Storage.SampleLoader;
import marf.Storage.StorageException;
import marf.util.Arrays;
import marf.util.Debug;
import marf.util.InvalidSampleFormatException;


/**
 * @author serguei
 *
 */
public class RawSampleLoader
extends SampleLoader
{
	protected ByteArrayFileReader oReader = null;

	protected byte[] atBuffer = null;

	protected int iChunkNumber = 0;

	/**
	 *
	 */
	public RawSampleLoader()
	throws InvalidSampleFormatException
	{
		this.oSample = new Sample(MARFAudioFileFormat.CUSTOM);
		this.oSample.setSampleSize(Sample.DEFAULT_SAMPLE_SIZE);
		Debug.debug("RawSampleLoader constructed.");
	}

	/**
	 * Reads of the next chunk of character data as amplitudes.
	 * @param padSampleData to fill in with sample data
	 * @return the length of padSampleData
	 * @throws StorageException
	 */
	public final int readSampleData(double[] padSampleData)
	throws StorageException
	{
		try
		{
//			char[] acCharBuffer = new char[padSampleData.length];
			if(this.atBuffer == null)
			{
				this.atBuffer = this.oReader.toByteArray();
			}

			Debug.debug("RawSampleLoader: read length: " + this.atBuffer);

			if(this.atBuffer.length == 0)
			{
				throw new StorageException("Reader returned 0 bytes.");
			}

			if(Debug.isDebugOn())
			{
				for(int i = 0; i < 50; i++)
				{
					System.out.print("["+this.atBuffer[i]+"]");
				}
				System.out.println();
			}

			int iNbrDataItemsRead = 0;
//			int iNbrChars = this.oReader.read(acCharBuffer);
//			Debug.debug(getClass(), "read data " + iNbrChars + ", " + Arrays.arrayToCSV(acCharBuffer));

			//padSampleData = new double[atCharBuffer.length];
			padSampleData = new double[Sample.DEFAULT_SAMPLE_SIZE];

			Arrays.copy(padSampleData, 0, this.atBuffer, iChunkNumber * Sample.DEFAULT_SAMPLE_SIZE, Sample.DEFAULT_SAMPLE_SIZE);
			iNbrDataItemsRead = padSampleData.length;

			this.oSample.setSampleArray(padSampleData);

			return iNbrDataItemsRead;
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new StorageException(e);
		}
	}

	/**
	 * TODO: Does not do any actual writing.
	 * @param padSample used to verify the length
	 * @param piLength used to verify the length
	 * @return the smallest of padSample.length or piLength
	 * @throws StorageException if piLength is less than zero
	 */
	public final int writeSampleData(final double[] padSample, final int piLength)
	throws StorageException
	{
		int iSampleArrayLength = padSample == null ? 0 : padSample.length;

		if(piLength < 0)
		{
			throw new StorageException("Parameter length of a sample should not be < 0.");
		}

		return iSampleArrayLength < piLength ? iSampleArrayLength : piLength;
	}

	/**
	 * @see marf.Storage.ISampleLoader#loadSample(java.io.InputStream)
	 */
	public Sample loadSample(InputStream poDataInputStream)
	throws StorageException
	{
		try
		{
			this.oReader = new ByteArrayFileReader(poDataInputStream);
			//this.oSample.setSampleSize(this.oReader.getFileSize());
			Debug.debug(getClass(), "Constructed a reader");
			updateSample();
			Debug.debug(getClass(), "Updated a sample");
			Debug.debug(this.oSample);
			if(Debug.isDebugOn())
			{
				System.out.print("Length: " + this.oSample.getSampleArray().length + ">");
				for(int i = 0; i < 50; i++)
				{
					System.out.print("<" + this.oSample.getSampleArray()[i] + ">");
				}
				System.out.println();
			}
			return this.oSample;
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new StorageException(e);
		}
	}
}

// EOFL
